/*
 * $Id: DSMChartReader.h 1113 2008-10-14 19:41:23Z sayer $
 *
 * Copyright (C) 2008 iptego GmbH
 *
 * This file is part of SEMS, a free SIP media server.
 *
 * sems is free software; you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
 * the Free Software Foundation; either version 2 of the License, or
 * (at your option) any later version
 *
 * For a license to use the SEMS software under conditions
 * other than those described here, or to purchase support for this
 * software, please contact iptel.org by e-mail at the following addresses:
 *    info@iptel.org
 *
 * SEMS is distributed in the hope that it will be useful,
 * but WITHOUT ANY WARRANTY; without even the implied warranty of
 * MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 * GNU General Public License for more details.
 *
 * You should have received a copy of the GNU General Public License 
 * along with this program; if not, write to the Free Software 
 * Foundation, Inc., 59 Temple Place, Suite 330, Boston, MA  02111-1307  USA
 */
#ifndef _DSMChartReader_h_
#define _DSMChartReader_h_

#include "DSMStateEngine.h"
#include "DSMCoreModule.h"
#include "DSMElemContainer.h"

#include <string>
using std::string;

class NamedAction : public DSMAction {
 public:
  NamedAction(const string& m_name) {
    name = m_name;
  }
  void execute(AmSession* sess) { };
};

class AttribInitial : public DSMElement {
 public:
  AttribInitial() { }
};

class AttribName : public DSMElement {
 public:
  AttribName(const string& m_name) { name = m_name; }
};

class ActionList : public DSMElement {
 public:
  enum AL_type {
    AL_enter,
    AL_exit,
    AL_trans
  };

  AL_type al_type;
  
 ActionList(AL_type al_type) 
   : al_type(al_type) { }

  vector<DSMAction*> actions;
};

struct DSMConditionList : public DSMElement {
 DSMConditionList() { }
  vector<DSMCondition*> conditions;
};

class DSMChartReader {

  bool is_wsp(const char c);
  bool is_snt(const char c);

  string getToken(string str, size_t& pos);
  DSMAction* actionFromToken(const string& str);
  DSMCondition* conditionFromToken(const string& str);

  bool importModule(const string& mod_cmd, const string& mod_path);
  vector<DSMModule*> mods;
  DSMCoreModule core_mod;

 public:
  DSMChartReader();
  ~DSMChartReader();
  bool decode(DSMStateDiagram* e, const string& chart, 
	      const string& mod_path, DSMElemContainer* owner,
	      vector<DSMModule*>& out_mods);

  friend class DSMFactory;
};

#endif
